==> SimpleMailer-0.80.tar.bz2: SimpleMailer-0.80/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client/ImapOpenedMailboxInfo.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2011 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;

#if NET_3_5
using System.Linq;
#else
using Smdn.Collections;
#endif
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

namespace Smdn.Net.Imap4.Client {
  public partial class ImapOpenedMailboxInfo : ImapMailboxInfo, IDisposable {
    public bool IsReadOnly {
      get { return Mailbox.ReadOnly; }
    }

    public long FirstUnseenMessageNumber {
      get { return Mailbox.FirstUnseen; }
    }

    public IImapMessageFlagSet ApplicableFlags {
      get { return Mailbox.ApplicableFlags; }
    }

    public IImapMessageFlagSet PermanentFlags {
      get { return Mailbox.PermanentFlags; }
    }

    public bool IsAllowedToCreateKeywords {
      get { return Mailbox.PermanentFlags.Contains(ImapMessageFlag.AllowedCreateKeywords); }
    }

    public bool IsUidPersistent {
      get { return Mailbox.UidPersistent; }
    }

    private ImapMessageInfoCollection messages = new ImapMessageInfoCollection();

    internal ImapOpenedMailboxInfo(ImapClient client, ImapMailbox mailbox)
      : base(client, mailbox)
    {
    }

    void IDisposable.Dispose()
    {
      if (Client.IsConnected)
        Close();
    }

    public override ImapOpenedMailboxInfo Open(bool asReadOnly)
    {
      if (IsOpen)
        return this;
      else
        // messages.Clear();
        throw new NotImplementedException();
    }

    public void Close()
    {
      Client.CloseMailbox();
    }

    internal void CheckSelected()
    {
      if (!Client.IsSelected(this))
        throw new ImapMailboxClosedException(FullName);
    }

    internal void CheckUidValidity(long uidValidity, ImapSequenceSet sequenceOrUidSet)
    {
      // TODO: impl
#if false
      if (!sequenceOrUidSet.IsUidSet)
        return;
      if (uidValidity != UidValidity)
        throw new ImapException("UIDVALIDITY value has been changed");
#endif
    }

    internal ImapMessageInfo ToMessageInfo(ImapMessageAttribute message, bool hasStaticAttr, bool hasDynamicAttr)
    {
      ImapMessageInfo info;

      if (messages.Contains(message.Uid)) {
        info = messages[message.Uid];
        info.Sequence = message.Sequence;
      }
      else {
        info = new ImapMessageInfo(this, message.Uid, message.Sequence);

        messages.Add(info);
      }

      if (hasStaticAttr)
        info.StaticAttribute = message.GetStaticAttributeImpl();
      if (hasDynamicAttr)
        info.DynamicAttribute = message.GetDynamicAttributeImpl();

      return info;
    }

    internal protected override ImapCommandResult ProcessResult(ImapCommandResult result,
                                                                Func<ImapResponseCode, bool> throwIfError)
    {
      ProcessSizeAndStatusResponse(result.ReceivedResponses);

      return base.ProcessResult(result, throwIfError);
    }

    private void ProcessSizeAndStatusResponse(IEnumerable<ImapResponse> receivedResponses)
    {
      var prevExistMessageCount = Mailbox.ExistsMessage;
      var prevRecentMessageCount = Mailbox.RecentMessage;
      var deletedMessages = new List<ImapMessageInfo>();
      var statusChangedMessages = new List<ImapMessageInfo>();

      foreach (var response in receivedResponses) {
        var data = response as ImapDataResponse;

        if (data == null)
          continue;

        // XXX: equality
        if (data.Type == ImapDataResponseType.Fetch) {
          var message = messages.Find(ImapDataResponseConverter.FromFetch(messages, data));

          if (message != null)
            statusChangedMessages.Add(message);
        }
        else if (data.Type == ImapDataResponseType.Expunge) {
          var expunged = ImapDataResponseConverter.FromExpunge(data);

          foreach (var message in messages) {
            if (message.Sequence == expunged) {
              message.Sequence = ImapMessageInfo.ExpungedMessageSequenceNumber;

              deletedMessages.Add(message);
            }
            else if (expunged < message.Sequence) {
              message.Sequence--;
            }
          }

          if (0L < Mailbox.ExistsMessage)
            Mailbox.ExistsMessage -= 1L;
        }
        else if (data.Type == ImapDataResponseType.Exists) {
          Mailbox.ExistsMessage = ImapDataResponseConverter.FromExists(data);
        }
        else if (data.Type == ImapDataResponseType.Recent) {
          Mailbox.RecentMessage = ImapDataResponseConverter.FromRecent(data);
        }
        else if (data.Type == ImapDataResponseType.Flags) {
          Mailbox.ApplicableFlags = ImapDataResponseConverter.FromFlags(data);
        }
      }

      if (prevExistMessageCount != Mailbox.ExistsMessage)
        Client.RaiseExistMessageCountChanged(this, prevExistMessageCount);

      if (prevRecentMessageCount != Mailbox.RecentMessage)
        Client.RaiseRecentMessageCountChanged(this, prevRecentMessageCount);

      if (0 < statusChangedMessages.Count)
        Client.RaiseMessageStatusChanged(statusChangedMessages.ToArray());

      if (0 < deletedMessages.Count)
        Client.RaiseMessageDeleted(deletedMessages.ToArray());
    }

    /*
     * operations
     */
    public override void Refresh()
    {
      CheckSelected();

      ProcessResult(Client.Session.NoOp());
    }

    public void Expunge()
    {
      CheckSelected();

      ProcessResult(Client.Session.Expunge());
    }

    /*
     * MoveMessagesTo
     */
    public void MoveMessagesTo(ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void MoveMessagesTo(ImapMailboxInfo destinationMailbox,
                               long uid,
                               params long[] uids)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(ImapMessageFetchAttributeOptions.None,
                                           uid,
                                           uids),
                               destinationMailbox);
    }

    public void MoveMessagesTo(ImapSearchCriteria searchCriteria,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(searchCriteria,
                                           null,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void MoveMessagesTo(ImapSearchCriteria searchCriteria,
                               Encoding encoding,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(searchCriteria,
                                           encoding,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    /*
     * CopyMessagesTo
     */
    public void CopyMessagesTo(ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void CopyMessagesTo(ImapMailboxInfo destinationMailbox,
                               long uid,
                               params long[] uids)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(ImapMessageFetchAttributeOptions.None,
                                           uid,
                                           uids),
                               destinationMailbox);
    }

    public void CopyMessagesTo(ImapSearchCriteria searchCriteria,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(searchCriteria,
                                           null,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void CopyMessagesTo(ImapSearchCriteria searchCriteria,
                               Encoding encoding,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(searchCriteria,
                                           encoding,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    private void MoveOrCopyMessagesToCore(bool copy,
                                          ImapMessageInfoList messages,
                                          ImapMailboxInfo destinationMailbox)
    {
      if (messages == null)
        throw new ArgumentNullException("messages");
      if (destinationMailbox == null)
        throw new ArgumentNullException("destinationMailbox");

      if (copy)
        messages.CopyTo(destinationMailbox);
      else
        messages.MoveTo(destinationMailbox);
    }
  }
}

==> SimpleMailer-0.90.tar.bz2: SimpleMailer-0.90/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client/ImapOpenedMailboxInfo.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2013 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;

#if NET_3_5
using System.Linq;
#else
using Smdn.Collections;
#endif
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

namespace Smdn.Net.Imap4.Client {
  public partial class ImapOpenedMailboxInfo : ImapMailboxInfo, IDisposable {
    public override bool IsOpen {
      get { return Client.OpenedMailbox == this; }
    }

    public bool IsReadOnly {
      get { return Mailbox.ReadOnly; }
    }

    public long FirstUnseenMessageNumber {
      get { return Mailbox.FirstUnseen; }
    }

    public IImapMessageFlagSet ApplicableFlags {
      get { return Mailbox.ApplicableFlags; }
    }

    public IImapMessageFlagSet PermanentFlags {
      get { return Mailbox.PermanentFlags; }
    }

    public bool IsAllowedToCreateKeywords {
      get { return Mailbox.PermanentFlags.Contains(ImapMessageFlag.AllowedCreateKeywords); }
    }

    public bool IsUidPersistent {
      get { return Mailbox.UidPersistent; }
    }

    private ImapMessageInfoCollection messages = new ImapMessageInfoCollection();

    internal ImapOpenedMailboxInfo(ImapClient client, ImapMailbox mailbox)
      : base(client, mailbox)
    {
    }

    void IDisposable.Dispose()
    {
      if (Client.IsConnected && IsOpen)
        Client.CloseMailbox();
    }

    public override ImapOpenedMailboxInfo Open(bool asReadOnly)
    {
      if (IsOpen)
        return this;
      else
        // messages.Clear();
        throw new NotImplementedException();
    }

    public void Close()
    {
      if (IsOpen)
        Client.CloseMailbox();
    }

    internal void CheckSelected()
    {
      if (!IsOpen)
        throw new ImapUnavailableException();
    }

    internal void CheckUidValidity(long uidValidity, ImapSequenceSet sequenceOrUidSet)
    {
      // TODO: impl
#if false
      if (!sequenceOrUidSet.IsUidSet)
        return;
      if (uidValidity != UidValidity)
        throw new ImapException("UIDVALIDITY value has been changed");
#endif
    }

    internal ImapMessageInfo ToMessageInfo(ImapMessageAttribute message, bool hasStaticAttr, bool hasDynamicAttr)
    {
      var info = messages.FindByUid(message.Uid);

      if (info == null) {
        info = new ImapMessageInfo(this, message.Uid, message.Sequence);

        messages.Add(info);
      }
      else if (info.Sequence != message.Sequence) {
        // XXX: this would be never occur?
        messages.ChangeSequenceNumber(info, message.Sequence);
      }

      if (hasStaticAttr)
        info.StaticAttribute = message.GetStaticAttributeImpl();
      if (hasDynamicAttr)
        info.DynamicAttribute = message.GetDynamicAttributeImpl();

      return info;
    }

    internal void Detach(ImapMessageInfoBase mailboxObject)
    {
      mailboxObject.DetachFromMailbox();

      var message = mailboxObject as ImapMessageInfo;

      if (message != null)
        messages.Remove(message);
    }

    internal void DetachAllMailboxObject()
    {
      foreach (var message in messages) {
        message.DetachFromMailbox();
      }

      messages.Clear();
    }

    internal ImapCommandResult ProcessMessageStatusChangedResult(ImapCommandResult result,
                                                                 ImapMessageAttributeBase[] statusChangedMessageAttrs)
    {
      ProcessSizeAndStatusResponse(result.ReceivedResponses, false);

      if (statusChangedMessageAttrs != null) {
        RaiseMessageStatusChanged(statusChangedMessageAttrs.Select(delegate(ImapMessageAttributeBase attr) {
          return attr.Sequence;
        }));
      }

      return base.ProcessResult(result, null);
    }

    internal protected override ImapCommandResult ProcessResult(ImapCommandResult result,
                                                                Func<ImapResponseCode, bool> throwIfError)
    {
      ProcessSizeAndStatusResponse(result.ReceivedResponses, true);

      return base.ProcessResult(result, throwIfError);
    }

    private void ProcessSizeAndStatusResponse(IEnumerable<ImapResponse> receivedResponses, bool processFetchResponse)
    {
      var prevExistMessageCount = Mailbox.ExistsMessage;
      var prevRecentMessageCount = Mailbox.RecentMessage;
      List<ImapMessageInfo> deletedMessages = null;
      List<long> statusChangedSequenceNumbers = null;

      foreach (var response in receivedResponses) {
        var data = response as ImapDataResponse;

        if (data == null)
          continue;

        // XXX: equality
        if (processFetchResponse && data.Type == ImapDataResponseType.Fetch) {
          if (statusChangedSequenceNumbers == null)
            statusChangedSequenceNumbers = new List<long>();

          statusChangedSequenceNumbers.Add(ImapDataResponseConverter.FromFetch(messages, data));
        }
        else if (data.Type == ImapDataResponseType.Expunge) {
          var expunged = ImapDataResponseConverter.FromExpunge(data);

          foreach (var message in messages) {
            if (message.Sequence == expunged) {
              if (deletedMessages == null)
                deletedMessages = new List<ImapMessageInfo>();

              deletedMessages.Add(message);
            }
            else if (expunged < message.Sequence) {
              message.Sequence--;
            }
          }

          if (0L < Mailbox.ExistsMessage)
            Mailbox.ExistsMessage -= 1L;
        }
        else if (data.Type == ImapDataResponseType.Exists) {
          Mailbox.ExistsMessage = ImapDataResponseConverter.FromExists(data);
        }
        else if (data.Type == ImapDataResponseType.Recent) {
          Mailbox.RecentMessage = ImapDataResponseConverter.FromRecent(data);
        }
        else if (data.Type == ImapDataResponseType.Flags) {
          Mailbox.ApplicableFlags = ImapDataResponseConverter.FromFlags(data);
        }
      }

      if (deletedMessages != null) {
        foreach (var message in deletedMessages) {
          message.SetDeleted();
        }
      }

      if (prevExistMessageCount != Mailbox.ExistsMessage)
        Client.RaiseExistMessageCountChanged(this, prevExistMessageCount);

      if (prevRecentMessageCount != Mailbox.RecentMessage)
        Client.RaiseRecentMessageCountChanged(this, prevRecentMessageCount);

      if (statusChangedSequenceNumbers != null)
        RaiseMessageStatusChanged(statusChangedSequenceNumbers);

      if (deletedMessages != null)
        Client.RaiseMessageDeleted(deletedMessages.ToArray());
    }

    private void RaiseMessageStatusChanged(IEnumerable<long> statusChangedSequenceNumbers)
    {
      var statusChangedMessages = new List<ImapMessageInfo>();

      foreach (var sequenceNumber in statusChangedSequenceNumbers) {
        var message = messages.FindBySequence(sequenceNumber);

        if (message != null)
          statusChangedMessages.Add(message);
      }

      Client.RaiseMessageStatusChanged(statusChangedMessages.ToArray());
    }

    /*
     * operations
     */
    public override void Refresh()
    {
      CheckSelected();

      ProcessResult(Client.Session.NoOp());
    }

    public void Expunge()
    {
      CheckSelected();

      ProcessResult(Client.Session.Expunge());
    }

    /*
     * MoveMessagesTo
     */
    public void MoveMessagesTo(ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void MoveMessagesTo(ImapMailboxInfo destinationMailbox,
                               long uid,
                               params long[] uids)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(ImapMessageFetchAttributeOptions.None,
                                           uid,
                                           uids),
                               destinationMailbox);
    }

    public void MoveMessagesTo(ImapSearchCriteria searchCriteria,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(searchCriteria,
                                           null,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void MoveMessagesTo(ImapSearchCriteria searchCriteria,
                               Encoding encoding,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(false,
                               GetMessages(searchCriteria,
                                           encoding,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    /*
     * CopyMessagesTo
     */
    public void CopyMessagesTo(ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void CopyMessagesTo(ImapMailboxInfo destinationMailbox,
                               long uid,
                               params long[] uids)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(ImapMessageFetchAttributeOptions.None,
                                           uid,
                                           uids),
                               destinationMailbox);
    }

    public void CopyMessagesTo(ImapSearchCriteria searchCriteria,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(searchCriteria,
                                           null,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    public void CopyMessagesTo(ImapSearchCriteria searchCriteria,
                               Encoding encoding,
                               ImapMailboxInfo destinationMailbox)
    {
      MoveOrCopyMessagesToCore(true,
                               GetMessages(searchCriteria,
                                           encoding,
                                           ImapMessageFetchAttributeOptions.None),
                               destinationMailbox);
    }

    private void MoveOrCopyMessagesToCore(bool copy,
                                          ImapMessageInfoList messages,
                                          ImapMailboxInfo destinationMailbox)
    {
      if (messages == null)
        throw new ArgumentNullException("messages");
      if (destinationMailbox == null)
        throw new ArgumentNullException("destinationMailbox");

      if (copy)
        messages.CopyTo(destinationMailbox);
      else
        messages.MoveTo(destinationMailbox);
    }
  }
}
